using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Collections;

namespace EtchASketch
{
    public partial class Form1 : Form
    {
        private ArrayList alphaPath;
        private Stack<Point> path = null;
        private Bitmap bmpBack = null;
        private Point currentPosition;
        private float lineWidth = 1.0f;
        private float leftKnobAngle = 0.0f;
        private float rightKnobAngle = 0.0f;
        private Bitmap leftKnobImage = null;
        private Bitmap rightKnobImage = null;
        private Keys keyHoriz = Keys.None;
        private Keys keyVert = Keys.None;

        public Form1()
        {
            InitializeComponent();

            //Backup images for knob image rotating
            leftKnobImage = (Bitmap)leftKnob.Image.Clone();
            rightKnobImage = (Bitmap)rightKnob.Image.Clone();

            //Ini all our vars to keep track of the sketch
            path = new Stack<Point>();
            bmpBack = new Bitmap(image.Width, image.Height);
            Graphics.FromImage(bmpBack).Clear(Color.LightGray);
            image.Image = (Bitmap)bmpBack.Clone();

            this.SetStyle(ControlStyles.DoubleBuffer, true);
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            reset();
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (eraseTimer.Enabled)
                return;

            bool dirty = false;

            switch (e.KeyCode)
            {
                case Keys.Down:
                    //currentPosition.Y++;
                    //dirty = true;
                    //RotateImage(rightKnob, rightKnobImage, rightKnobAngle++);
                    keyVert = e.KeyCode;
                    break;
                case Keys.Up:
                    //currentPosition.Y--;
                    //dirty = true;
                    //RotateImage(rightKnob, rightKnobImage, rightKnobAngle--);
                    keyVert = e.KeyCode;
                    break;
                case Keys.Right:
                    //currentPosition.X++;
                    //dirty = true;
                    //RotateImage(leftKnob, leftKnobImage, leftKnobAngle++);
                    keyHoriz = e.KeyCode;
                    break;
                case Keys.Left:
                    //currentPosition.X--;
                    //dirty = true;
                    //RotateImage(leftKnob, leftKnobImage, leftKnobAngle--);
                    keyHoriz = e.KeyCode;
                    break;
            }

            switch (keyHoriz)
            {
                case Keys.Right:
                    currentPosition.X++;
                    dirty = true;
                    RotateImage(leftKnob, leftKnobImage, leftKnobAngle++);
                    break;
                case Keys.Left:
                    currentPosition.X--;
                    dirty = true;
                    RotateImage(leftKnob, leftKnobImage, leftKnobAngle--);
                    break;
            }

            switch (keyVert)
            {
                case Keys.Down:
                    currentPosition.Y++;
                    dirty = true;
                    RotateImage(rightKnob, rightKnobImage, rightKnobAngle++);
                    break;
                case Keys.Up:
                    currentPosition.Y--;
                    dirty = true;
                    RotateImage(rightKnob, rightKnobImage, rightKnobAngle--);
                    break;
            }

            if(dirty)
                updatePath();
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (eraseTimer.Enabled)
                return;

            //Ctrl+Z
            if (e.KeyCode == Keys.Z && e.Control)
            {
                //Undo previous Move.

                if (path.Count >= 1)
                {
                    currentPosition = path.Pop();
                    Redraw(currentPosition);
                }
            }

            if (e.KeyCode == keyVert)
            {
                keyVert = Keys.None;
            }

            if (e.KeyCode == keyHoriz)
            {
                keyHoriz = Keys.None;
            }
        }

        private void updatePath()
        {
            if (eraseTimer.Enabled)
                return;

            if (currentPosition.X < 0)
            {
                currentPosition.X = 0;
                return;
            }
            else if (currentPosition.X > image.Width)
            {
                currentPosition.X = image.Width;
                return;
            }
            else if (currentPosition.Y < 0)
            {
                currentPosition.Y = 0;
                return;
            }
            else if (currentPosition.Y > image.Height)
            {
                currentPosition.Y = image.Height;
                return;
            }

            path.Push(currentPosition);
            Redraw(currentPosition);
        }

        //Exit
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sfd.InitialDirectory = Application.StartupPath;
            sfd.ShowDialog();
        }

        //Saves file
        private void sfd_FileOk(object sender, CancelEventArgs e)
        {
            try
            {
                image.Image.Save(sfd.FileName, ImageFormat.Jpeg);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error");
            }
        }

        private void resetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clear();
            reset();
        }

        //Shake until clear
        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            alphaPath = new ArrayList();
            for (int i = 0; i < path.Count; i++)
            {
                alphaPath.Insert(i, 255);
            }
            eraseTimer.Start();
        }

        private void reset()
        {
            currentPosition.X = image.Width / 2;
            currentPosition.Y = image.Height / 2;
            leftKnobAngle = 0.0f;
            rightKnobAngle = 0.0f;
            keyHoriz = Keys.None;
            keyVert = Keys.None;

            RotateImage(leftKnob, leftKnobImage, leftKnobAngle);
            RotateImage(rightKnob, rightKnobImage, rightKnobAngle);
        }

        private void clear()
        {
            image.Controls.Clear();
            bmpBack = new Bitmap(image.Width, image.Height);
            image.Image = (Bitmap)bmpBack.Clone();

            path.Clear();
        }

        private void eraseTimer_Tick(object sender, EventArgs e)
        {
            if (shake())
            {
                alphaPath.Clear();
                path.Clear();
                eraseTimer.Stop();
            }
        }

        private bool shake()
        {
            bool drewPoint = false;
            int alpha = 0;
            Random number = new Random();
            if (bmpBack != null)
            {
                image.Image = (Bitmap)bmpBack.Clone();
            }
            else
            {
                image.Image = new Bitmap(image.Width, image.Height);
                Graphics.FromImage(image.Image).Clear(Color.Transparent);
            }

            int i = 0;
            foreach (Point p in path)
            {
                alpha = number.Next(0, (int)alphaPath[i]);
                alphaPath[i] = alpha;
                if (alpha > 0)
                {
                    drewPoint = true;
                    DrawPoint(p, Color.FromArgb(alpha, Color.Black));
                }
                i++;
            }
            image.Refresh();

            return !drewPoint;
        }

        //Draw Cursor
        private void DrawCursor(Point pt)
        {
            DrawPoint(pt, Color.White);
        }

        //Simply draws a point
        private void DrawPoint(Point pt)
        {
            DrawPoint(pt, Color.Black);
        }

        private void DrawPoint(Point pt, Color c)
        {
            Graphics g = null;

            g = Graphics.FromImage(image.Image);
            g.DrawRectangle(new Pen(c, lineWidth), pt.X, pt.Y, 1, 1);
            //g.DrawLine(new Pen(Color.Black, lineWidth), start, end);
            g.Dispose();
        }

        //Redraws all the lines and a part of the background
        private void Redraw(Point pt)
        {
            Graphics.FromImage(image.Image).DrawImage(bmpBack, 0, 0, image.Image.Width,
                image.Image.Height);

            foreach (Point p in path)
            {
                DrawPoint(p);
            }

            DrawCursor(pt);

            image.Invalidate(new Rectangle(pt.X-4, pt.Y-4, 8, 8));
            image.Update();
        }

        private void Redraw()
        {
            if (bmpBack != null)
            {
                image.Image = (Bitmap)bmpBack.Clone();
            }
            else
            {
                image.Image = new Bitmap(image.Width, image.Height);
                Graphics.FromImage(image.Image).Clear(Color.Transparent);
            }

            foreach (Point p in path)
            {
                DrawPoint(p);
            }
            image.Refresh();
        }

        private void RotateImage(PictureBox pb, Image img, float angle)
        {
            Image oldImage = pb.Image;
            pb.Image = Utilities.RotateImage(img, angle);
            if (oldImage != null)
            {
                oldImage.Dispose();
            }
        }
    }
}